<?php

namespace Drupal\filehash_report\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\Component\Render\FormattableMarkup;
use Drupal\Component\Utility\HTML;

/**
 * Provides a Filehash report form.
 */
class DuplicatesForm extends FormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'filehash_report_duplicates';
  }


  /**
   *  Return sha256 for fid.
   **/
  public function getHashByFid($fid) {
    $database = \Drupal::database();
    $result = $database->query('select sha256 from filehash where fid = :fid', [ ':fid' => $fid ])
      ->fetchAllAssoc('sha256', \PDO::FETCH_ASSOC);
    $sha256s = array_keys($result);
    $sha256 = reset($sha256s);
    return $sha256;
  }

  /**
   *  Return all fids for sha256 value.
   **/
  public function getFidsBySha256($sha256) {
    $database = \Drupal::database();
    $result = $database->query('select fid from filehash where sha256 = :sha256', [ ':sha256' => $sha256 ])
      ->fetchAllAssoc('fid', \PDO::FETCH_ASSOC);
    $fids = array_keys($result);
    return $fids;
  }

  /**
   *  Return array of fid.
   **/
  public function getDuplicateFids() {
    $database = \Drupal::database();
    $result = $database->query('select fid from filehash group by sha256 having count(sha256) > 1')
      ->fetchAllAssoc('fid', \PDO::FETCH_ASSOC);
    $fids = array_keys($result);
    return $fids;
  }

  public function buildHtmlTable() {
    $fids = $this->getDuplicateFids();
    $table_html = '';
    $table_html = '<table class="file-hash table">';
    $table_html .= '<thead>';
    $table_html .= '<td>' . t('FID action') . '</td>';
    $table_html .= '<td>' . t('FID usage') . '</td>';
    $table_html .= '<td>' . t('File name') . '</td>';
    $table_html .= '<td>' . t('File uri') . '</td>';
    $table_html .= '<td>' . t('Sha256') . '</td>';
    $table_html .= '</thead>';
    $html = '';
    $base_path = Url::fromRoute('<front>')->toString();
    $report_path = Url::fromRoute('filehash.reportduplicates')->toString();
    foreach ($fids as $fid) {
      $sha256 = $this->getHashByFid($fid);
      $fids_with_matches = $this->getFidsBySha256($sha256); 
      foreach ($fids_with_matches as $fid_matched) {
        $table_html .= '<tr>';
        $table_html .= '<td>';
        $table_html .= "<a href='" . $base_path . "/admin/content/files/replace/$fid_matched?destination=$report_path'>" . $fid_matched . "</a><br>";
        $table_html .= '</td>';
        $table_html .= '<td>';
        $table_html .= "<a href='" . $base_path . "/admin/content/files/usage/$fid_matched?destination=$report_path' target='_blank'>" . $fid_matched . "</a><br>";
        $table_html .= '</td>';
        $file = \Drupal\file\Entity\File::load($fid_matched);
        $filename = $file->getFilename();
        $table_html .= '<td>';
	$limited_filename = HTML::escape($filename);
	$limited_filename = substr($filename, 0 , 70);
        $table_html .= $limited_filename;
        $table_html .= '</td>';
        $file_uri = str_replace('public://', '', $file->getFileUri());
	$file_uri = (string) HTML::escape($file_uri);
        $table_html .= '<td>';
        $table_html .= "<a href='" . $base_path . '/../sites/default/files/' . $file_uri . "'>" . substr($file_uri, 0, 35) . "</a>";
        $table_html .= '</td>';
        $table_html .= '<td>';
	$table_html .= substr($sha256, 0, 8) . '...';
        $table_html .= '</td>';
        $table_html .= '</tr>';
      }
    }
    $table_html .= '</table>';
    $table_html .= '<div>' . count($fids) . ' ' . t('duplicates in total') . '</div>';
    return $table_html;
    return $html;
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    $html = '<strong>Test</strong>';
    $args = [];
    $html = $this->buildHtmlTable();
    $html_formatted = new FormattableMarkup($html, $args);
    $form['message'] = [
      '#type' => 'markup',
      '#title' => $this->t('Message'),
      '#markup' => $html_formatted,
    ];

    $form['actions'] = [
      '#type' => 'actions',
    ];
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Re-generate file hashes'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
//    if (mb_strlen($form_state->getValue('message')) < 10) {
//      $form_state->setErrorByName('message', $this->t('Message should be at least 10 characters.'));
//    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->messenger()->addStatus($this->t('The filehashes are regenerating.'));
    $form_state->setRedirect('filehash.reportduplicates');
  }

}
